@@ -1,11 +1,29 @@
 /*======================================================================
 
-    PCMCIA Card Manager
+    PCMCIA Card Manager daemon
 
-    Written by David Hinds, dhinds@hyper.stanford.edu
+    cardmgr.c 1.114 1998/08/09 00:15:41
+
+    The contents of this file are subject to the Mozilla Public
+    License Version 1.0 (the "License"); you may not use this file
+    except in compliance with the License. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
+
+    Software distributed under the License is distributed on an "AS
+    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+    implied. See the License for the specific language governing
+    rights and limitations under the License.
+
+    The initial developer of the original code is David A. Hinds
+    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
     
 ======================================================================*/
 
+#ifndef __linux__
+#include <pcmcia/u_compat.h>
+#endif
+
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -21,7 +39,6 @@
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/utsname.h>
-#include <sys/kd.h>
 #include <sys/file.h>
 
 #include <pcmcia/version.h>
@@ -92,8 +109,10 @@ static char *configpath = "/etc/pcmcia";
 /* Default path for pid file */
 static char *pidfile = "/var/run/cardmgr.pid";
 
+#ifdef __linux__
 /* Default path for finding modules */
 static char *modpath = NULL;
+#endif
 
 /* Default path for socket info table */
 static char *stabfile = "/var/run/stab";
@@ -110,8 +129,15 @@ static int one_pass = 0;
 /* most recent signal */
 static int caught_signal = 0;
 
+/* Extra message logging? */
+static int verbose = 0;
+
 /*====================================================================*/
 
+#ifdef __linux__
+
+static int major = 0;
+
 static int lookup_dev(char *name)
 {
     FILE *f;
@@ -133,13 +159,14 @@ static int lookup_dev(char *name)
 	return -ENODEV;
 }
 
+#endif /* __linux__ */
+
 /*====================================================================*/
 
 int open_dev(dev_t dev, int mode)
 {
-    static char *fn;
+    char *fn;
     int fd;
-
     if ((fn = tmpnam(NULL)) == NULL)
 	return -1;
     if (mknod(fn, mode, dev) != 0)
@@ -149,6 +176,20 @@ int open_dev(dev_t dev, int mode)
     return fd;
 }
 
+int open_sock(int sock, int mode)
+{
+#ifdef __linux__
+    dev_t dev = (major<<8)+sock;
+    return open_dev(dev, mode);
+#endif
+#ifdef __BEOS__
+    int fd;
+    char fn[B_OS_NAME_LENGTH];
+    sprintf(fn, "/dev/pcmcia/sock%d", sock);
+    return open(fn, (mode & S_IWRITE) ? O_RDWR: O_RDONLY);
+#endif
+}
+
 /*======================================================================
 
     xlate_scsi_name() is a sort-of-hack used to deduce the minor
@@ -206,6 +247,10 @@ static int xlate_scsi_name(bind_info_t *bind)
 #define BEEP_WARN 2000
 #define BEEP_ERR  4000
 
+#ifdef __linux__
+
+#include <sys/kd.h>
+
 static void beep(unsigned int ms, unsigned int freq)
 {
     int fd, arg;
@@ -221,6 +266,15 @@ static void beep(unsigned int ms, unsigned int freq)
     usleep(ms*1000);
 }
 
+#endif /* __linux__ */
+
+#ifdef __BEOS__
+static void beep(unsigned int ms, unsigned int freq)
+{
+    system("/bin/beep");
+}
+#endif
+
 /*====================================================================*/
 
 static void write_pid(void)
@@ -247,10 +301,12 @@ static void write_stab(void)
 	syslog(LOG_INFO, "fopen(stabfile) failed: %m");
 	return;
     }
+#ifndef __BEOS__
     if (flock(fileno(f), LOCK_EX) != 0) {
 	syslog(LOG_INFO, "flock(stabfile) failed: %m");
 	return;
     }
+#endif
     for (i = 0; i < sockets; i++) {
 	s = &socket[i];
 	if (!(s->state & SOCKET_PRESENT))
@@ -275,7 +331,9 @@ static void write_stab(void)
 	}
     }
     fflush(f);
+#ifndef __BEOS__
     flock(fileno(f), LOCK_UN);
+#endif
     fclose(f);
 }
 
@@ -303,6 +361,35 @@ static int get_tuple(int ns, cisdata_t code, ds_ioctl_arg_t *arg)
 
 /*====================================================================*/
 
+static void log_card_info(cistpl_vers_1_t *vers,
+			  cistpl_manfid_t *manfid,
+			  cistpl_funcid_t *funcid)
+{
+    char v[256] = "";
+    int i;
+    static char *fn[] = {
+	"multi", "memory", "serial", "parallel", "fixed disk",
+	"video", "network", "AIMS", "SCSI"
+    };
+    
+    if (vers) {
+	for (i = 0; i < vers->ns; i++)
+	    sprintf(v+strlen(v), "%s\"%s\"",
+		    (i>0) ? ", " : "", vers->str+vers->ofs[i]);
+	syslog(LOG_INFO, "  product info: %s", v);
+    } else {
+	syslog(LOG_INFO, "  no product info available");
+    }
+    *v = '\0';
+    if (manfid->manf != 0)
+	sprintf(v, "  manfid: 0x%04x, 0x%04x",
+		manfid->manf, manfid->card);
+    if (funcid->func != 0xff)
+	sprintf(v+strlen(v), "  function: %d (%s)", funcid->func,
+		fn[funcid->func]);
+    if (strlen(v) > 0) syslog(LOG_INFO, "%s", v);
+}
+
 static card_info_t *lookup_card(int ns)
 {
     socket_info_t *s = &socket[ns];
@@ -310,6 +397,7 @@ static card_info_t *lookup_card(int ns)
     ds_ioctl_arg_t arg;
     cistpl_vers_1_t *vers = NULL;
     cistpl_manfid_t manfid = { 0, 0 };
+    cistpl_funcid_t funcid = { 0xff, 0xff };
     int i, ret, match;
     int has_cis = 0;
 
@@ -319,6 +407,9 @@ static card_info_t *lookup_card(int ns)
     
     /* Try to read VERS_1, MANFID tuples */
     if (has_cis) {
+	if (get_tuple(ns, CISTPL_FUNCID, &arg) == 0)
+	    memcpy(&funcid, &arg.tuple_parse.parse.funcid,
+		   sizeof(funcid));
 	if (get_tuple(ns, CISTPL_MANFID, &arg) == 0)
 	    memcpy(&manfid, &arg.tuple_parse.parse.manfid,
 		   sizeof(manfid));
@@ -360,7 +451,8 @@ static card_info_t *lookup_card(int ns)
 		arg.tuple.TupleOffset = card->id.tuple.ofs;
 		ret = ioctl(s->fd, DS_GET_TUPLE_DATA, &arg);
 		if (ret != 0) break;
-		if (strncmp(arg.tuple_parse.data, card->id.tuple.info,
+		if (strncmp((char *)arg.tuple_parse.data,
+			    card->id.tuple.info,
 			    strlen(card->id.tuple.info)) != 0)
 		    break;
 		match = 1;
@@ -375,18 +467,20 @@ static card_info_t *lookup_card(int ns)
 	if (match) {
 	    syslog(LOG_INFO, "socket %d: %s", ns, card->name);
 	    beep(BEEP_TIME, BEEP_OK);
+	    if (verbose) log_card_info(vers, &manfid, &funcid);
 	    return card;
 	}
     }
 
     /* Try for a FUNCID match */
-    if (get_tuple(ns, CISTPL_FUNCID, &arg) == 0) {
+    if (funcid.func != 0xff) {
 	for (card = root_func; card; card = card->next)
-	    if (card->id.func.funcid == arg.tuple_parse.parse.funcid.func)
+	    if (card->id.func.funcid == funcid.func)
 		break;
 	if (card) {
 	    syslog(LOG_INFO, "socket %d: %s", ns, card->name);
 	    beep(BEEP_TIME, BEEP_OK);
+	    if (verbose) log_card_info(vers, &manfid, &funcid);
 	    return card;
 	}
     }
@@ -395,18 +489,7 @@ static card_info_t *lookup_card(int ns)
 	syslog(LOG_INFO, "unsupported card in socket %d", ns);
 	if (one_pass) return NULL;
 	beep(BEEP_TIME, BEEP_ERR);
-	if (vers) {
-	    char v[256] = "";
-	    for (i = 0; i < vers->ns; i++)
-		sprintf(v+strlen(v), "%s\"%s\"",
-			(i>0) ? ", " : "", vers->str+vers->ofs[i]);
-	    syslog(LOG_INFO, "version info: %s", v);
-	}
-	else
-	    syslog(LOG_INFO, "no version info");
-	if (manfid.manf != 0)
-	    syslog(LOG_INFO, "manfid: 0x%04x, 0x%04x",
-		   manfid.manf, manfid.card);
+	log_card_info(vers, &manfid, &funcid);
 	return NULL;
     } else {
 	card = blank_card;
@@ -426,7 +509,7 @@ static void load_config(void)
 	exit(EXIT_FAILURE);
     if (root_device == NULL)
 	syslog(LOG_INFO, "no device drivers defined");
-    if (root_card == NULL)
+    if ((root_card == NULL) && (root_func == NULL))
 	syslog(LOG_INFO, "no cards defined");
 }
 
@@ -514,8 +597,7 @@ static int execute(char *msg, char *cmd)
 	    syslog(LOG_INFO, "%s exited with status %d",
 		   msg, WEXITSTATUS(ret));
 	return WEXITSTATUS(ret);
-    }
-    else
+    } else
 	syslog(LOG_INFO, "%s exited on signal %d",
 	       msg, WTERMSIG(ret));
     return -1;
@@ -545,6 +627,8 @@ static int execute_on_all(char *cmd, char *class, int sn, int fn)
 
 /*====================================================================*/
 
+#ifdef __linux__
+
 typedef struct module_list_t {
     char *mod;
     int usage;
@@ -618,6 +702,24 @@ static void remove_module(char *mod)
     }
 }
 
+#endif /* __linux__ */
+
+/*====================================================================*/
+
+#ifdef __BEOS__
+
+#define install_module(a,b)
+#define remove_module(a)
+
+static void republish_driver(char *mod)
+{
+    int fd = open("/dev", O_RDWR);
+    write(fd, mod, strlen(mod));
+    close(fd);
+}
+
+#endif /* __BEOS__ */
+
 /*====================================================================*/
 
 static mtd_ident_t *lookup_mtd(region_info_t *region)
@@ -668,7 +770,7 @@ static void bind_mtd(int sn)
 		for (i = 0; i < nr; i++)
 		    if (strcmp(s->mtd[i], mtd->module) == 0) break;
 		if (i == nr) {
-		    install_module(mtd->module, NULL);
+		    install_module(mtd->module, mtd->opts);
 		    s->mtd[nr] = mtd->module;
 		    nr++;
 		}
@@ -693,7 +795,6 @@ static void bind_mtd(int sn)
     /* Now bind each unique MTD as a normal client of this socket */
     for (i = 0; i < nr; i++) {
 	strcpy(bind.dev_info, s->mtd[i]);
-	bind.instance = NULL;
 	if (ioctl(s->fd, DS_BIND_REQUEST, &bind) != 0)
 	    syslog(LOG_INFO, "bind MTD '%s' to socket %d failed: %m",
 		   (char *)bind.dev_info, sn);
@@ -702,6 +803,22 @@ static void bind_mtd(int sn)
 
 /*====================================================================*/
 
+static void update_cis(socket_info_t *s)
+{
+    cisdump_t cis;
+    FILE *f = fopen(s->card->cis_file, "r");
+    if (f == NULL)
+	syslog(LOG_INFO, "could not open '%s': %m", s->card->cis_file);
+    else {
+	cis.Length = fread(cis.Data, 1, CISTPL_MAX_CIS_SIZE, f);
+	fclose(f);
+	if (ioctl(s->fd, DS_REPLACE_CIS, &cis) != 0)
+	    syslog(LOG_INFO, "could not replace CIS: %m");
+    }
+}
+
+/*====================================================================*/
+
 static void do_insert(int sn)
 {
     socket_info_t *s = &socket[sn];
@@ -720,6 +837,7 @@ static void do_insert(int sn)
     if (card == s->card)
 	return;
     s->card = card;
+    if (card->cis_file) update_cis(s);
 
     dev = card->device;
 
@@ -729,13 +847,15 @@ static void do_insert(int sn)
 	    break;
     if (i < card->functions)
 	bind_mtd(sn);
-    
+
+#ifdef __linux__
     /* Install kernel modules */
     for (i = 0; i < card->functions; i++) {
 	for (j = 0; j < dev[i]->modules; j++)
 	    install_module(dev[i]->module[j], dev[i]->opts[j]);
     }
-
+#endif
+    
     /* Bind drivers by their dev_info identifiers */
     for (i = 0; i < card->functions; i++) {
 	bind = calloc(1, sizeof(bind_info_t));
@@ -756,6 +876,11 @@ static void do_insert(int sn)
 		return;
 	    }
 	}
+
+#ifdef __BEOS__
+	republish_driver(dev[i]->module[0]);
+#endif
+
 	for (j = 0; j < 10; j++) {
 	    ret = ioctl(s->fd, DS_GET_DEVICE_INFO, bind);
 	    if ((ret == 0) || (errno != EAGAIN))
@@ -860,6 +985,13 @@ static void do_remove(int sn)
 	    }
 	}
     }
+    for (i = 0; (s->mtd[i] != NULL); i++) {
+	bind_info_t b;
+	strcpy(b.dev_info, s->mtd[i]);
+	if (ioctl(s->fd, DS_UNBIND_REQUEST, &b) != 0)
+	    syslog(LOG_INFO, "unbind MTD '%s' from socket %d failed: %m",
+		   (char *)s->mtd[i], sn);
+    }
 
     /* remove kernel modules in inverse order */
     for (i = 0; i < card->functions; i++) {
@@ -924,7 +1056,7 @@ static void do_resume(int sn)
 
 static void wait_for_pending(void)
 {
-    status_t status;
+    cs_status_t status;
     int i;
     status.Function = 0;
     for (;;) {
@@ -948,9 +1080,8 @@ static void free_resources(void)
 	if (al->adj.Action == ADD_MANAGED_RESOURCE) {
 	    al->adj.Action = REMOVE_MANAGED_RESOURCE;
 	    ioctl(fd, DS_ADJUST_RESOURCE_INFO, &al->adj);
-	}
-	else if ((al->adj.Action == REMOVE_MANAGED_RESOURCE) &&
-		 (al->adj.Resource == RES_IRQ)) {
+	} else if ((al->adj.Action == REMOVE_MANAGED_RESOURCE) &&
+		   (al->adj.Resource == RES_IRQ)) {
 	    al->adj.Action = ADD_MANAGED_RESOURCE;
 	    ioctl(fd, DS_ADJUST_RESOURCE_INFO, &al->adj);
 	}
@@ -972,7 +1103,7 @@ static void adjust_resources(void)
 	if (ret != 0) {
 	    switch (al->adj.Resource) {
 	    case RES_MEMORY_RANGE:
-		sprintf(tmp, "memory %p-%p",
+		sprintf(tmp, "memory %#lx-%#lx",
 			al->adj.resource.memory.Base,
 			al->adj.resource.memory.Base +
 			al->adj.resource.memory.Size - 1);
@@ -1051,8 +1182,10 @@ static void handle_signal(void)
 	    if (socket[i].state & SOCKET_PRESENT)
 		do_insert(i);
 	break;
+#ifdef SIGPWR
     case SIGPWR:
 	break;
+#endif
     }
 }
 
@@ -1060,9 +1193,10 @@ static void handle_signal(void)
 
 static int init_sockets(void)
 {
-    int major, fd, i;
+    int fd, i;
     servinfo_t serv;
-    
+
+#ifdef __linux__
     major = lookup_dev("pcmcia");
     if (major < 0) {
 	if (major == -ENODEV)
@@ -1071,16 +1205,15 @@ static int init_sockets(void)
 	    syslog(LOG_INFO, "could not open /proc/devices: %m");
 	exit(EXIT_FAILURE);
     }
+#endif
     for (i = 0; i < MAX_SOCKS; i++) {
-	fd = open_dev((major<<8)+i, S_IFCHR|S_IREAD|S_IWRITE);
+	fd = open_sock(i, S_IFCHR|S_IREAD|S_IWRITE);
 	if (fd < 0) break;
 	socket[i].fd = fd;
 	socket[i].state = 0;
     }
-    if ((fd < 0) && (errno != ENODEV)) {
-	syslog(LOG_INFO, "open_dev(socket %d) failed: %m", i);
-	return -1;
-    }
+    if ((fd < 0) && (errno != ENODEV) && (errno != ENOENT))
+	syslog(LOG_INFO, "open_sock(socket %d) failed: %m", i);
     sockets = i;
     if (sockets == 0) {
 	syslog(LOG_INFO, "no sockets found!");
@@ -1105,14 +1238,17 @@ int main(int argc, char *argv[])
 {
     int optch, errflg;
     int i, max_fd, ret, event, pass;
-    int verbose = 0, delay_fork = 0;
+    int delay_fork = 0;
     struct timeval tv;
-    extern char *optarg;
     fd_set fds;
 
     errflg = 0;
-    while ((optch = getopt(argc, argv, "qdvofc:m:p:s:")) != -1) {
+    while ((optch = getopt(argc, argv, "Vqdvofc:m:p:s:")) != -1) {
 	switch (optch) {
+	case 'V':
+	    fprintf(stderr, "cardmgr version " CS_RELEASE "\n");
+	    return 0;
+	    break;
 	case 'q':
 	    be_quiet = 1; break;
 	case 'd':
@@ -1125,8 +1261,10 @@ int main(int argc, char *argv[])
 	    delay_fork = 1; break;
 	case 'c':
 	    configpath = strdup(optarg); break;
+#ifdef __linux__
 	case 'm':
 	    modpath = strdup(optarg); break;
+#endif
 	case 'p':
 	    pidfile = strdup(optarg); break;
 	case 's':
@@ -1136,7 +1274,7 @@ int main(int argc, char *argv[])
 	}
     }
     if (errflg || (optind < argc)) {
-	fprintf(stderr, "usage: %s [-q] [-v] [-d] [-o] [-f] "
+	fprintf(stderr, "usage: %s [-V] [-q] [-v] [-d] [-o] [-f] "
 		"[-c configpath] [-m modpath]\n               "
 		"[-p pidfile] [-s stabfile]\n", argv[0]);
 	exit(EXIT_FAILURE);
@@ -1145,10 +1283,7 @@ int main(int argc, char *argv[])
 #ifdef DEBUG
     openlog("cardmgr", LOG_PID|LOG_PERROR, LOG_DAEMON);
 #else
-    if (verbose)
-	openlog("cardmgr", LOG_PID|LOG_PERROR, LOG_DAEMON);
-    else
-	openlog("cardmgr", LOG_PID|LOG_CONS, LOG_DAEMON);
+    openlog("cardmgr", LOG_PID|LOG_CONS, LOG_DAEMON);
 #endif
 
 #ifndef DEBUG
@@ -1158,8 +1293,9 @@ int main(int argc, char *argv[])
     
     syslog(LOG_INFO, "starting, version is " CS_RELEASE);
     atexit(&done);
-    setenv("PATH", "/bin:/sbin:/usr/bin:/usr/sbin", 1);
-    
+    putenv("PATH=/bin:/sbin:/usr/bin:/usr/sbin");
+
+#ifdef __linux__
     if (modpath == NULL) {
 	struct utsname utsname;
 	if (uname(&utsname) != 0) {
@@ -1173,6 +1309,7 @@ int main(int argc, char *argv[])
 	syslog(LOG_INFO, "cannot access %s: %m", modpath);
 	exit(EXIT_FAILURE);
     }
+#endif /* __linux__ */
     
     load_config();
     
@@ -1189,9 +1326,11 @@ int main(int argc, char *argv[])
 	syslog(LOG_INFO, "signal(SIGTERM): %m");
     if (signal(SIGINT, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(SIGINT): %m");
+#ifdef SIGPWR
     if (signal(SIGPWR, catch_signal) == SIG_ERR)
 	syslog(LOG_INFO, "signal(SIGPWR): %m");
-
+#endif
+    
     for (i = max_fd = 0; i < sockets; i++)
 	max_fd = (socket[i].fd > max_fd) ? socket[i].fd : max_fd;
 
@@ -1209,9 +1348,9 @@ int main(int argc, char *argv[])
 
 	while ((ret = select(max_fd+1, &fds, NULL, NULL,
 			     ((pass == 0) ? &tv : NULL))) < 0) {
-	    if (errno == EINTR)
+	    if (errno == EINTR) {
 		handle_signal();
-	    else {
+	    } else {
 		syslog(LOG_INFO, "select(): %m");
 		exit(EXIT_FAILURE);
 	    }
@@ -1222,7 +1361,7 @@ int main(int argc, char *argv[])
 		continue;
 	    ret = read(socket[i].fd, &event, 4);
 	    if ((ret == -1) && (errno != EAGAIN))
-		syslog(LOG_INFO, "read(): %m\n");
+		syslog(LOG_INFO, "read(%d): %m\n", i);
 	    if (ret != 4)
 		continue;
 	    
