@@ -2,7 +2,21 @@
 
     Register dump for the Databook TCIC-2 controller family
 
-    Written by David Hinds, dhinds@allegro.stanford.edu
+    dump_tcic.c 1.16 1998/08/13 09:27:53
+
+    The contents of this file are subject to the Mozilla Public
+    License Version 1.0 (the "License"); you may not use this file
+    except in compliance with the License. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
+
+    Software distributed under the License is distributed on an "AS
+    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+    implied. See the License for the specific language governing
+    rights and limitations under the License.
+
+    The initial developer of the original code is David A. Hinds
+    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
     
 ======================================================================*/
 
@@ -38,9 +52,6 @@ typedef unsigned short u_short;
 
 #include "tcic.h"
 
-static const char *version =
-"dump_tcic.c 1.12 1998/01/17 05:13:26 (David Hinds)\n";
-
 /*====================================================================*/
 
 static int tcic_base = TCIC_BASE;
@@ -438,13 +449,10 @@ void dump_windows(void)
 
 /*====================================================================*/
 
-void main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int sock, i;
     
-    if ((argc > 1) && (strcmp(argv[1], "-v") == 0))
-	printf("%s", version);
-    
 #ifndef __MSDOS__
     ioperm(tcic_base, 16, 1);
     ioperm(0x80, 1, 1);
@@ -459,4 +467,5 @@ void main(int argc, char *argv[])
 	dump_sock(i);
     }
     dump_windows();
+    return 0;
 }
