@@ -1,5 +1,19 @@
 /*
- * cistpl.h 1.20 1998/01/02 02:07:30 (David Hinds)
+ * cistpl.h 1.26 1998/07/09 08:41:19
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_CISTPL_H
@@ -68,8 +82,8 @@ typedef struct cistpl_longlink_mfc_t {
 #define CISTPL_MAX_ALTSTR_STRINGS	4
 
 typedef struct cistpl_altstr_t {
-    int		ns;
-    int		ofs[CISTPL_MAX_ALTSTR_STRINGS];
+    u_char	ns;
+    u_char	ofs[CISTPL_MAX_ALTSTR_STRINGS];
     char	str[254];
 } cistpl_altstr_t;
 
@@ -87,7 +101,7 @@ typedef struct cistpl_altstr_t {
 #define CISTPL_MAX_DEVICES	4
 
 typedef struct cistpl_device_t {
-    int		ndev;
+    u_char	ndev;
     struct device_info {
 	u_char 	type;
 	u_char	wp;
@@ -109,13 +123,13 @@ typedef struct cistpl_device_o_t {
 typedef struct cistpl_vers_1_t {
     u_char	major;
     u_char	minor;
-    int		ns;
-    int		ofs[CISTPL_VERS_1_MAX_PROD_STRINGS];
+    u_char	ns;
+    u_char	ofs[CISTPL_VERS_1_MAX_PROD_STRINGS];
     char	str[254];
 } cistpl_vers_1_t;
 
 typedef struct cistpl_jedec_t {
-    int		nid;
+    u_char	nid;
     struct jedec_id {
 	u_char	mfr;
 	u_char	info;
@@ -333,7 +347,7 @@ typedef struct cistpl_ide_feature_t {
 
 typedef struct cistpl_bar_t {
     u_char	attr;
-    u_long	size;
+    u_int	size;
 } cistpl_bar_t;
 
 typedef struct cistpl_config_t {
@@ -376,7 +390,7 @@ typedef struct cistpl_timing_t {
 
 typedef struct cistpl_io_t {
     u_char	flags;
-    int		nwin;
+    u_char	nwin;
     struct {
 	u_int	base;
 	u_int	len;
@@ -396,7 +410,7 @@ typedef struct cistpl_mem_t {
     struct {
 	u_int	len;
 	u_int	card_addr;
-	caddr_t	host_addr;
+	u_int	host_addr;
     } win[CISTPL_MEM_MAX_WIN];
 } cistpl_mem_t;
 
@@ -442,7 +456,7 @@ typedef struct cistpl_cftable_entry_cb_t {
 } cistpl_cftable_entry_cb_t;
 
 typedef struct cistpl_device_geo_t {
-    int		ngeo;
+    u_char		ngeo;
     struct geo {
 	u_char		buswidth;
 	u_int		erase_block;
@@ -459,7 +473,7 @@ typedef struct cistpl_vers_2_t {
     u_short	dindex;
     u_char	vspec8, vspec9;
     u_char	nhdr;
-    int		vendor, info;
+    u_char	vendor, info;
     char	str[244];
 } cistpl_vers_2_t;
 
@@ -518,4 +532,12 @@ typedef struct cisinfo_t {
     u_int	Chains;
 } cisinfo_t;
 
+#define CISTPL_MAX_CIS_SIZE	0x200
+
+/* For ReplaceCIS */
+typedef struct cisdump_t {
+    u_int	Length;
+    cisdata_t	Data[CISTPL_MAX_CIS_SIZE];
+} cisdump_t;
+
 #endif /* LINUX_CISTPL_H */
