@@ -1,5 +1,19 @@
 /*
- *  cs.h 1.49 1997/12/29 17:04:50 (David Hinds)
+ * cs.h 1.60 1998/08/03 19:12:23
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_CS_H
@@ -24,7 +38,7 @@ typedef struct adjust_t {
     u_int	Attributes;
     union {
 	struct memory {
-	    caddr_t	Base;
+	    u_long	Base;
 	    u_long	Size;
 	} memory;
 	struct io {
@@ -111,26 +125,6 @@ typedef struct client_req_t {
 
 #define CLIENT_THIS_SOCKET	0x01
 
-/* For GetFirstRegion and GetNextRegion */
-typedef struct region_info_t {
-    u_int		Attributes;
-    u_int		CardOffset;
-    u_int		RegionSize;
-    u_int		AccessSpeed;
-    u_int		BlockSize;
-    u_int		PartMultiple;
-    u_char		JedecMfr, JedecInfo;
-    memory_handle_t	next;
-} region_info_t;
-
-#define REGION_TYPE		0x0001
-#define REGION_TYPE_CM		0x0000
-#define REGION_TYPE_AM		0x0001
-#define REGION_PREFETCH		0x0008
-#define REGION_CACHEABLE	0x0010
-#define REGION_BAR_MASK		0xe000
-#define REGION_BAR_SHIFT	13
-
 /* For RegisterClient */
 typedef struct client_reg_t {
     dev_info_t	*dev_info;
@@ -265,7 +259,7 @@ typedef struct modwin_t {
 /* For RequestWindow */
 typedef struct win_req_t {
     u_int	Attributes;
-    caddr_t	Base;
+    u_long	Base;
     u_int	Size;
     u_int	AccessSpeed;
 } win_req_t;
@@ -301,11 +295,11 @@ typedef struct win_req_t {
 #define INFO_CARD_SHARE		0x10
 #define INFO_CARD_EXCL		0x20
 
-typedef struct status_t {
+typedef struct cs_status_t {
     u_char	Function;
     event_t 	CardState;
     event_t	SocketState;
-} status_t;
+} cs_status_t;
 
 typedef struct error_info_t {
     int		func;
@@ -354,6 +348,8 @@ typedef struct mtd_bind_t {
 #define CS_EVENT_ERASE_COMPLETE		0x040000
 #define CS_EVENT_REQUEST_ATTENTION	0x080000
 #define CS_EVENT_CB_DETECT		0x100000
+#define CS_EVENT_3VCARD			0x200000
+#define CS_EVENT_XVCARD			0x400000
 
 /* Return codes */
 #define CS_SUCCESS		0x00
@@ -420,7 +416,7 @@ enum service {
     RequestSocketMask, RequestWindow, ResetCard, ReturnSSEntry,
     SetEventMask, SetRegion, ValidateCIS, VendorSpecific,
     WriteMemory, BindDevice, BindMTD, ReportError,
-    SuspendCard, ResumeCard, EjectCard, InsertCard
+    SuspendCard, ResumeCard, EjectCard, InsertCard, ReplaceCIS
 };
 
 #ifdef IN_CARD_SERVICES
@@ -429,6 +425,25 @@ extern int CardServices(int func, void *a1, void *a2, void *a3);
 extern int CardServices(int func, ...);
 #endif
 
+#ifdef __BEOS__
+#define CS_CLIENT_MODULE_NAME "bus_managers/pcmcia_cs/client/v1"
+typedef struct cs_client_module_info {
+    bus_manager_info	binfo;
+    int (*_CardServices)(int, ...);
+    int (*_MTDHelperEntry)(int, ...);
+    void (*_add_timer)(struct timer_list *);
+    void (*_del_timer)(struct timer_list *);
+} cs_client_module_info;
+#define CS_SOCKET_MODULE_NAME "bus_managers/pcmcia_cs/socket/v1"
+typedef struct cs_socket_module_info {
+    bus_manager_info	binfo;
+    int (*_register_ss_entry)(int, ss_entry_t);
+    void (*_unregister_ss_entry)(ss_entry_t);
+    void (*_add_timer)(struct timer_list *);
+    void (*_del_timer)(struct timer_list *);
+} cs_socket_module_info;
+#endif
+
 #endif /* __KERNEL__ */
 
 #endif /* _LINUX_CS_H */
