@@ -1,15 +1,30 @@
 /*
- * ds.h 1.36 1998/02/03 06:17:19 (David Hinds)
+ * ds.h 1.49 1998/08/03 19:12:23
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_DS_H
 #define _LINUX_DS_H
 
 #include <pcmcia/driver_ops.h>
+#include <pcmcia/bulkmem.h>
 
 typedef struct tuple_parse_t {
     tuple_t		tuple;
-    char		data[255];
+    cisdata_t		data[255];
     cisparse_t		parse;
 } tuple_parse_t;
 
@@ -17,7 +32,7 @@ typedef struct bind_info_t {
     dev_info_t		dev_info;
     u_char		function;
     struct dev_link_t	*instance;
-    char		name[8];
+    char		name[DEV_NAME_LEN];
     u_short		major, minor;
     void		*next;
 } bind_info_t;
@@ -35,12 +50,13 @@ typedef union ds_ioctl_arg_t {
     tuple_t		tuple;
     tuple_parse_t	tuple_parse;
     client_req_t	client_req;
-    status_t		status;
+    cs_status_t		status;
     conf_reg_t		conf_reg;
     cisinfo_t		cisinfo;
     region_info_t	region;
     bind_info_t		bind_info;
     mtd_info_t		mtd_info;
+    cisdump_t		cisdump;
 } ds_ioctl_arg_t;
 
 #define DS_GET_CARD_SERVICES_INFO	_IOR ('d', 1, servinfo_t)
@@ -51,7 +67,7 @@ typedef union ds_ioctl_arg_t {
 #define DS_GET_TUPLE_DATA		_IOWR('d', 6, tuple_parse_t)
 #define DS_PARSE_TUPLE			_IOWR('d', 7, tuple_parse_t)
 #define DS_RESET_CARD			_IO  ('d', 8)
-#define DS_GET_STATUS			_IOWR('d', 9, status_t)
+#define DS_GET_STATUS			_IOWR('d', 9, cs_status_t)
 #define DS_ACCESS_CONFIGURATION_REGISTER _IOWR('d', 10, conf_reg_t)
 #define DS_VALIDATE_CIS			_IOR ('d', 11, cisinfo_t)
 #define DS_SUSPEND_CARD			_IO  ('d', 12)
@@ -60,6 +76,7 @@ typedef union ds_ioctl_arg_t {
 #define DS_INSERT_CARD			_IO  ('d', 15)
 #define DS_GET_FIRST_REGION		_IOWR('d', 16, region_info_t)
 #define DS_GET_NEXT_REGION		_IOWR('d', 17, region_info_t)
+#define DS_REPLACE_CIS			_IOWR('d', 18, cisdump_t)
 
 #define DS_BIND_REQUEST			_IOWR('d', 60, bind_info_t)
 #define DS_GET_DEVICE_INFO		_IOWR('d', 61, bind_info_t) 
@@ -96,14 +113,29 @@ typedef struct dev_link_t {
 #define DEV_OK(l) \
     ((l) && ((l->state & ~DEV_BUSY) == (DEV_CONFIG|DEV_PRESENT)))
 
-int register_pcmcia_driver(dev_info_t *dev_info,
+int register_pccard_driver(dev_info_t *dev_info,
 			   dev_link_t *(*attach)(void),
 			   void (*detach)(dev_link_t *));
 
-int unregister_pcmcia_driver(dev_info_t *dev_info);
+int unregister_pccard_driver(dev_info_t *dev_info);
 
-int bind_request(int i, bind_info_t *bind_info);
-int bind_mtd(int i, mtd_info_t *mtd_info);
+#define register_pcmcia_driver register_pccard_driver
+#define unregister_pcmcia_driver unregister_pccard_driver
+
+#ifdef __BEOS__
+#define DS_MODULE_NAME "bus_managers/pcmcia_ds/v1"
+typedef struct ds_module_info {
+    bus_manager_info binfo;
+    int (*_register_pccard_driver)(dev_info_t *,
+				   dev_link_t *(*)(void),
+				   void (*)(dev_link_t *));
+    int (*_unregister_pccard_driver)(dev_info_t *);
+    struct driver_info_t **root_driver;
+    int *sockets;
+    struct socket_info_t **socket_table;
+    sem_id *list_sem;
+} ds_module_info;
+#endif /* __BEOS__ */
 
 #endif /* __KERNEL__ */
 
