@@ -1,10 +1,25 @@
 /*
- * k_compat.h 1.42 1998/02/12 00:01:20 (David Hinds)
+ * k_compat.h 1.59 1998/08/14 10:00:45
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_K_COMPAT_H
 #define _LINUX_K_COMPAT_H
 
+#define __LINUX__
 #define VERSION(v,p,s)		(((v)<<16)+(p<<8)+s)
 
 #if (LINUX_VERSION_CODE < VERSION(1,3,0))
@@ -141,14 +156,15 @@ char kernel_version[] = UTS_RELEASE;
 #if (LINUX_VERSION_CODE < VERSION(1,3,0))
 #define kdev_t			int
 #define ioremap(a, b)		((char *)(a))
-#define iounmap(a)
+#define iounmap(a)		while (0)
 #define put_user(x, ptr) \
 		((sizeof(*ptr) == 4) ? put_fs_long(x, ptr) : \
 		 (sizeof(*ptr) == 2) ? put_fs_word(x, ptr) : \
 		 put_fs_byte(x, ptr))
 #else
-#define ioremap			vremap
-#define iounmap			vfree
+#define ioremap(a,b)		(((a) < 0x100000) ? (void *)(a) : vremap(a,b))
+#define iounmap(v)		do { if ((u_long)(v) > 0x100000) \
+				     vfree(v); } while (0)
 /* This is evil... throw away the built-in get_user in 1.3, 2.0 */
 #include <asm/segment.h>
 #undef get_user
@@ -204,9 +220,82 @@ char kernel_version[] = UTS_RELEASE;
 #endif
 
 #if (LINUX_VERSION_CODE < VERSION(2,1,86))
-#define DEV_KFREE_SKB(skb) dev_kfree_skb(skb, FREE_WRITE)
+#define DEV_KFREE_SKB(skb)	dev_kfree_skb(skb, FREE_WRITE)
 #else
-#define DEV_KFREE_SKB(skb) dev_kfree_skb(skb)
+#define DEV_KFREE_SKB(skb)	dev_kfree_skb(skb)
 #endif
 
+#if (LINUX_VERSION_CODE < VERSION(2,1,89))
+#define POLL_WAIT(f, q, w)	poll_wait(q, w)
+#else
+#define POLL_WAIT(f, q, w)	poll_wait(f, q, w)
+#endif
+
+#include <asm/byteorder.h>
+#ifndef le16_to_cpu
+#define le16_to_cpu(x)		(x)
+#define le32_to_cpu(x)		(x)
+#define cpu_to_le16(x)		(x)
+#define cpu_to_le32(x)		(x)
+#endif
+
+#if (LINUX_VERSION_CODE < VERSION(2,1,90))
+#define spin_lock_irqsave(l,f) do { save_flags(f); cli(); } while (0)
+#define spin_unlock_irqrestore(l,f) do { restore_flags(f); } while (0)
+#else
+#include <asm/spinlock.h>
+#endif
+
+#if (LINUX_VERSION_CODE < VERSION(2,1,104))
+#define mdelay(x) { int i; for (i=0;i<x;i++) udelay(1000); }
+#endif
+
+#define wacquire(w)		do { } while (0)
+#define wrelease(w)		do { } while (0)
+#define wsleep(w)		interruptible_sleep_on(w)
+#define wsleeptimeout(w,t) \
+    do { current->timeout = jiffies+(t); wsleep(w); } while (0);
+#define wakeup(w)		wake_up_interruptible(w)
+
+#include <asm/io.h>
+#ifndef readw_ns
+#ifdef __powerpc__
+#define readw_ns(p)		ld_be16((volatile unsigned short *)(p))
+#define readl_ns(p)		ld_be32((volatile unsigned *)(p))
+#define writew_ns(v,p)		st_le16((volatile unsigned short *)(p),(v))
+#define writel_ns(v,p)		st_le32((volatile unsigned *)(p),(v))
+#define inw_ns(p)		in_be16((unsigned short *)((p)+_IO_BASE))
+#define inl_ns(p)		in_be32((unsigned *)((p)+_IO_BASE))
+#define outw_ns(v,p)		out_be16((unsigned short *)((p)+_IO_BASE),(v))
+#define outl_ns(v,p)		out_be32((unsigned *)((p)+_IO_BASE),(v))
+#else
+#define readw_ns(p)		readw(p)
+#define readl_ns(p)		readl(p)
+#define writew_ns(v,p)		writew(v,p)
+#define writel_ns(v,p)		writel(v,p)
+#define inw_ns(p)		inw(p)
+#define inl_ns(p)		inl(p)
+#define outw_ns(v,p)		outw(v,p)
+#define outl_ns(v,p)		outl(v,p)
+#endif
+#endif
+#ifndef insw_ns
+#define insw_ns(p,b,l)		insw(p,b,l)
+#define insl_ns(p,b,l)		insl(p,b,l)
+#define outsw_ns(p,b,l)		outsw(p,b,l)
+#define outsl_ns(p,b,l)		outsl(p,b,l)
+#endif
+
+#if (LINUX_VERSION_CODE < VERSION(2,1,93))
+#include <linux/bios32.h>
+#endif
+#include <linux/pci.h>
+#ifndef PCI_FUNC
+#define PCI_FUNC(devfn)		((devfn)&7)
+#define PCI_SLOT(devfn)		((devfn)>>3)
+#define PCI_DEVFN(dev,fn)	(((dev)<<3)|((fn)&7))
+#endif
+
+typedef unsigned long k_time_t;
+
 #endif /* _LINUX_K_COMPAT_H */
