@@ -2,7 +2,21 @@
 
     PCMCIA Card Information Structure parser
 
-    Written by David Hinds, dhinds@allegro.stanford.edu
+    cistpl.c 1.57 1998/08/09 00:16:16
+
+    The contents of this file are subject to the Mozilla Public
+    License Version 1.0 (the "License"); you may not use this file
+    except in compliance with the License. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
+
+    Software distributed under the License is distributed on an "AS
+    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+    implied. See the License for the specific language governing
+    rights and limitations under the License.
+
+    The initial developer of the original code is David A. Hinds
+    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
     
 ======================================================================*/
 
@@ -10,6 +24,7 @@
 #define __NO_VERSION__
 #include <pcmcia/k_compat.h>
 
+#ifdef __LINUX__
 #include <linux/kernel.h>
 #include <linux/string.h>
 #include <linux/major.h>
@@ -18,10 +33,8 @@
 #include <linux/malloc.h>
 #include <linux/mm.h>
 #include <linux/sched.h>
+#include <linux/pci.h>
 #include <asm/io.h>
-
-#ifdef CONFIG_CARDBUS
-#include <linux/bios32.h>
 #endif
 
 #include <pcmcia/cs_types.h>
@@ -32,7 +45,7 @@
 #include "cs_internal.h"
 #include "rsrc_mgr.h"
 
-#define MIN(a, b) (((a) < (b)) ? (a) : (b))
+#define MIN(a, b)		(((a) < (b)) ? (a) : (b))
 
 static const u_char mantissa[] = {
     10, 12, 13, 15, 20, 25, 30, 35,
@@ -49,10 +62,10 @@ static const u_int exponent[] = {
 /* Convert a power byte to a current in 0.1 microamps */
 #define POWER_CVT(v) \
     (mantissa[((v)>>3)&15] * exponent[(v)&7] / 10)
-#define POWER_SCALE(v) (exponent[(v)&7])
+#define POWER_SCALE(v)		(exponent[(v)&7])
 
 /* Upper limit on reasonable # of tuples */
-#define MAX_TUPLES 200
+#define MAX_TUPLES		200
 
 /*======================================================================
 
@@ -64,65 +77,61 @@ static const u_int exponent[] = {
 void read_cis_mem(socket_info_t *s, int attr, u_int addr,
 		  u_int len, void *ptr)
 {
-    pcmcia_mem_map *mem = &s->cis_mem;
+    pccard_mem_map *mem = &s->cis_mem;
     u_char *sys;
     u_int inc = 1;
     
-    DEBUG(3, "cs: read_cis_mem(%d, %#x, %u)\n", attr, addr, len);
+    DEBUG(3, ("cs: read_cis_mem(%d, %#x, %u)\n", attr, addr, len));
+    if (setup_cis_mem(s) != 0) {
+	memset(ptr, 0xff, len);
+	return;
+    }
     mem->flags &= ~MAP_ATTRIB;
     if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
-    sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
-    mem->card_start = addr & ~0x0fff;
-    
-    for (; len > 0; mem->card_start += 0x1000, sys -= 0x1000) {
+    sys = s->cis_virt + (addr & (s->cap.map_size-1));
+    mem->card_start = addr & ~(s->cap.map_size-1);
+
+    for (; len > 0; sys = s->cis_virt) {
 	s->ss_entry(s->sock, SS_SetMemMap, mem);
-	DEBUG(3,  "cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
-	      readb(sys), readb(sys+inc), readb(sys+2*inc),
-	      readb(sys+3*inc), readb(sys+4*inc));
-	for ( ; len > 0; len--, ptr++, sys += inc) {
-	    if (sys > (u_char *)mem->sys_stop) break;
+	DEBUG(3,  ("cs:  %#2.2x %#2.2x %#2.2x %#2.2x %#2.2x ...\n",
+		   readb(sys), readb(sys+inc), readb(sys+2*inc),
+		   readb(sys+3*inc), readb(sys+4*inc)));
+	for ( ; len > 0; len--, ((u_char *)ptr)++, sys += inc) {
+	    if (sys == s->cis_virt+s->cap.map_size) break;
 	    *(u_char *)ptr = readb(sys);
 	}
+	mem->card_start += s->cap.map_size;
     }
 }
 
-#ifdef CONFIG_CARDBUS
-void read_cb_mem(socket_info_t *s, u_char fn, int space, u_int addr,
-		 u_int len, void *ptr)
-{
-    DEBUG(3, "cs: read_cb_mem(%d, %#x, %u)\n", space, addr, len);
-    if (space == 0) {
-	for (; len; addr++, ptr++, len--)
-	    pcibios_read_config_byte(s->cap.cardbus, fn, addr, ptr);
-    }
-}
-#endif
-
 void write_cis_mem(socket_info_t *s, int attr, u_int addr,
 		   u_int len, void *ptr)
 {
-    pcmcia_mem_map *mem = &s->cis_mem;
+    pccard_mem_map *mem = &s->cis_mem;
     u_char *sys;
     int inc = 1;
     
+    DEBUG(3, ("cs: write_cis_mem(%d, %#x, %u)\n", attr, addr, len));
+    if (setup_cis_mem(s) != 0) return;
     mem->flags &= ~MAP_ATTRIB;
     if (attr) { mem->flags |= MAP_ATTRIB; inc++; addr *= 2; }
-    sys = (u_char *)(mem->sys_start + (addr & 0x0fff));
-    mem->card_start = addr & ~0x0fff;
+    sys = s->cis_virt + (addr & (s->cap.map_size-1));
+    mem->card_start = addr & ~(s->cap.map_size-1);
     
-    for (; len > 0; mem->card_start += 0x1000, sys -= 0x1000) {
+    for (; len > 0; sys = s->cis_virt) {
 	s->ss_entry(s->sock, SS_SetMemMap, mem);
-	for ( ; len > 0; len--, ptr++, sys += inc) {
-	    if (sys > (u_char *)mem->sys_stop) break;
+	for ( ; len > 0; len--, ((u_char *)ptr)++, sys += inc) {
+	    if (sys == s->cis_virt+s->cap.map_size) break;
 	    writeb(*(u_char *)ptr, sys);
 	}
+	mem->card_start += s->cap.map_size;
     }
 }
 
 /*======================================================================
 
     This is tricky... when we set up CIS memory, we try to validate
-    the memory window space below 1 MB.
+    the memory window space allocations.
     
 ======================================================================*/
 
@@ -135,15 +144,18 @@ static int cis_readable(u_long base)
     cisinfo_t info1, info2;
     int ret;
     vs->cis_mem.sys_start = base;
-    vs->cis_mem.sys_stop = base+0x0fff;
+    vs->cis_mem.sys_stop = base+vs->cap.map_size-1;
+    vs->cis_virt = ioremap(base, vs->cap.map_size);
     ret = validate_cis(vs->clients, &info1);
-    vs->cis_used = 0; /* invalidate CIS cache */
+    /* invalidate mapping and CIS cache */
+    iounmap(vs->cis_virt); vs->cis_used = 0;
     if ((ret != 0) || (info1.Chains == 0))
 	return 0;
-    vs->cis_mem.sys_start = base+0x1000;
-    vs->cis_mem.sys_stop = base+0x1fff;
+    vs->cis_mem.sys_start = base+vs->cap.map_size;
+    vs->cis_mem.sys_stop = base+2*vs->cap.map_size-1;
+    vs->cis_virt = ioremap(base+vs->cap.map_size, vs->cap.map_size);
     ret = validate_cis(vs->clients, &info2);
-    vs->cis_used = 0;
+    iounmap(vs->cis_virt); vs->cis_used = 0;
     return ((ret == 0) && (info1.Chains == info2.Chains));
 }
 
@@ -152,37 +164,49 @@ static int checksum(u_long base)
 {
     int i, a;
     vs->cis_mem.sys_start = base;
-    vs->cis_mem.sys_stop = base+0xfff;
+    vs->cis_mem.sys_stop = base+vs->cap.map_size-1;
+    vs->cis_virt = ioremap(base, vs->cap.map_size);
     vs->cis_mem.card_start = 0;
     vs->cis_mem.flags = MAP_ACTIVE;
     vs->ss_entry(vs->sock, SS_SetMemMap, &vs->cis_mem);
     /* Don't bother checking every word... */
-    for (i = a = 0; i < 0x1000; i += 56)
-	a += (int)readl(base+i);
+    for (i = a = 0; i < vs->cap.map_size; i += 56)
+	a += (int)readl(vs->cis_virt+i);
+    iounmap(vs->cis_virt);
     return a;
 }
 
 static int checksum_match(u_long base)
 {
-    return (checksum(base) == checksum(base+0x1000));
+    return (checksum(base) == checksum(base+vs->cap.map_size));
 }
 
 int setup_cis_mem(socket_info_t *s)
 {
     if (s->cis_mem.sys_start == 0) {
-	if (vs == NULL) {
-	    vs = s;
-	    validate_mem(cis_readable, checksum_match);
-	    s->cis_mem.sys_start = 0;
-	}
-	if (find_mem_region(&s->cis_mem.sys_start,
-			    0x1000, "card services") != 0)
+	int low = (s->cap.cardbus == 0);
+	vs = s;
+	validate_mem(cis_readable, checksum_match, low);
+	s->cis_mem.sys_start = 0;
+	vs = NULL;
+	if (find_mem_region(&s->cis_mem.sys_start, s->cap.map_size,
+			    "card services", low) != 0)
 	    return CS_OUT_OF_RESOURCE;
-	s->cis_mem.sys_stop = s->cis_mem.sys_start + 0x0fff;
+	s->cis_mem.sys_stop = s->cis_mem.sys_start+s->cap.map_size-1;
+	s->cis_virt = ioremap(s->cis_mem.sys_start, s->cap.map_size);
     }
     return 0;
 }
 
+void release_cis_mem(socket_info_t *s)
+{
+    if (s->cis_mem.sys_start != 0) {
+	release_mem_region(s->cis_mem.sys_start, s->cap.map_size);
+	iounmap(s->cis_virt);
+	s->cis_mem.sys_start = 0;
+    }
+}
+
 /*======================================================================
 
     This is a wrapper around read_cis_mem, with the same interface,
@@ -197,6 +221,13 @@ static void read_cis_cache(socket_info_t *s, int attr, u_int addr,
     int i;
     char *caddr;
 
+    if (s->fake_cis) {
+	if (s->fake_cis_len > addr+len)
+	    memcpy(ptr, s->fake_cis+addr, len);
+	else
+	    memset(ptr, 0xff, len);
+	return;
+    }
     caddr = s->cis_cache;
     for (i = 0; i < s->cis_used; i++) {
 	if ((s->cis_table[i].addr == addr) &&
@@ -239,8 +270,14 @@ int verify_cis_cache(socket_info_t *s)
     
     caddr = s->cis_cache;
     for (i = 0; i < s->cis_used; i++) {
-	read_cis_mem(s, s->cis_table[i].attr, s->cis_table[i].addr,
-		     s->cis_table[i].len, buf);
+#ifdef CONFIG_CARDBUS
+	if (s->state & SOCKET_CARDBUS)
+	    read_cb_mem(s, 0, s->cis_table[i].attr, s->cis_table[i].addr,
+			s->cis_table[i].len, buf);
+	else
+#endif
+	    read_cis_mem(s, s->cis_table[i].attr, s->cis_table[i].addr,
+			 s->cis_table[i].len, buf);
 	if (memcmp(buf, caddr, s->cis_table[i].len) != 0)
 	    break;
 	caddr += s->cis_table[i].len;
@@ -250,7 +287,34 @@ int verify_cis_cache(socket_info_t *s)
 
 /*======================================================================
 
-    Read tuple codes from attribute memory.
+    For really bad cards, we provide a facility for uploading a
+    replacement CIS.
+    
+======================================================================*/
+
+int replace_cis(client_handle_t handle, cisdump_t *cis)
+{
+    socket_info_t *s;
+    if (CHECK_HANDLE(handle))
+	return CS_BAD_HANDLE;
+    s = SOCKET(handle);
+    if (s->fake_cis != NULL) {
+	kfree(s->fake_cis);
+	s->fake_cis = NULL;
+    }
+    if (cis->Length > CISTPL_MAX_CIS_SIZE)
+	return CS_BAD_SIZE;
+    s->fake_cis = kmalloc(cis->Length, GFP_KERNEL);
+    if (s->fake_cis == NULL)
+	return CS_OUT_OF_RESOURCE;
+    s->fake_cis_len = cis->Length;
+    memcpy(s->fake_cis, cis->Data, cis->Length);
+    return CS_SUCCESS;
+}
+
+/*======================================================================
+
+    The high-level CIS tuple services
     
 ======================================================================*/
 
@@ -274,23 +338,23 @@ int get_first_tuple(client_handle_t handle, tuple_t *tuple)
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
     s = SOCKET(handle);
+    if (!(s->state & SOCKET_PRESENT))
+	return CS_NO_CARD;
+    tuple->TupleLink = tuple->Flags = 0;
 #ifdef CONFIG_CARDBUS
     if (s->state & SOCKET_CARDBUS) {
 	u_int ptr;
 	pcibios_read_config_dword(s->cap.cardbus, 0, 0x28, &ptr);
 	tuple->CISOffset = ptr & ~7;
-	tuple->TupleLink = 0;
 	SPACE(tuple->Flags) = (ptr & 7);
     } else {
-	tuple->CISOffset = tuple->TupleLink = 0;
 	/* Assume presence of a LONGLINK_C to address 0 */
-	tuple->LinkOffset = tuple->Flags = 0;
+	tuple->CISOffset = tuple->LinkOffset = 0;
 	SPACE(tuple->Flags) = HAS_LINK(tuple->Flags) = 1;
     }
 #else
-    tuple->CISOffset = tuple->TupleLink = 0;
     /* Assume presence of a LONGLINK_C to address 0 */
-    tuple->LinkOffset = tuple->Flags = 0;
+    tuple->CISOffset = tuple->LinkOffset = 0;
     SPACE(tuple->Flags) = HAS_LINK(tuple->Flags) = 1;
 #endif
     if (!(s->state & SOCKET_CARDBUS) && (s->functions > 1) &&
@@ -317,7 +381,7 @@ static int follow_link(socket_info_t *s, tuple_t *tuple)
 	/* Get indirect link from the MFC tuple */
 	read_cis_cache(s, LINK_SPACE(tuple->Flags),
 		       tuple->LinkOffset, 5, link);
-	ofs = *(u_int *)(link+1);
+	ofs = le32_to_cpu(*(u_int *)(link+1));
 	SPACE(tuple->Flags) = (link[0] == CISTPL_MFC_ATTR);
 	/* Move to the next indirect link */
 	tuple->LinkOffset += 5;
@@ -354,10 +418,9 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
     
     if (CHECK_HANDLE(handle))
 	return CS_BAD_HANDLE;
-
     s = SOCKET(handle);
-    if (setup_cis_mem(s) != 0)
-	return CS_OUT_OF_RESOURCE;
+    if (!(s->state & SOCKET_PRESENT))
+	return CS_NO_CARD;
 
     link[1] = tuple->TupleLink;
     ofs = tuple->CISOffset + tuple->TupleLink;
@@ -427,8 +490,8 @@ int get_next_tuple(client_handle_t handle, tuple_t *tuple)
 	ofs += link[1] + 2;
     }
     if (i == MAX_TUPLES) {
-	DEBUG(1, "cs: overrun in get_next_tuple for socket %d\n",
-	      handle->Socket);
+	DEBUG(1, ("cs: overrun in get_next_tuple for socket %d\n",
+		  handle->Socket));
 	return CS_NO_MORE_ITEMS;
     }
     
@@ -449,8 +512,6 @@ int get_tuple_data(client_handle_t handle, tuple_t *tuple)
 	return CS_BAD_HANDLE;
 
     s = SOCKET(handle);
-    if (setup_cis_mem(s) != 0)
-	return CS_OUT_OF_RESOURCE;
 
     if (tuple->TupleLink < tuple->TupleOffset)
 	return CS_NO_MORE_ITEMS;
@@ -519,12 +580,12 @@ static int parse_device(tuple_t *tuple, cistpl_device_t *device)
 
 static int parse_checksum(tuple_t *tuple, cistpl_checksum_t *csum)
 {
-    char *p;
+    u_char *p;
     if (tuple->TupleDataLen < 5)
 	return CS_BAD_TUPLE;
-    p = (char *)tuple->TupleData;
-    csum->addr = *(u_short *)p;
-    csum->len = *(u_short *)(p + 2);
+    p = (u_char *)tuple->TupleData;
+    csum->addr = tuple->CISOffset+(short)le16_to_cpu(*(u_short *)p)-2;
+    csum->len = le16_to_cpu(*(u_short *)(p + 2));
     csum->sum = *(p+4);
     return CS_SUCCESS;
 }
@@ -535,7 +596,7 @@ static int parse_longlink(tuple_t *tuple, cistpl_longlink_t *link)
 {
     if (tuple->TupleDataLen < 4)
 	return CS_BAD_TUPLE;
-    link->addr = *(u_int *)tuple->TupleData;
+    link->addr = le32_to_cpu(*(u_int *)tuple->TupleData);
     return CS_SUCCESS;
 }
 
@@ -547,14 +608,14 @@ static int parse_longlink_mfc(tuple_t *tuple,
     u_char *p;
     int i;
     
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     
     link->nfn = *p; p++;
     if (tuple->TupleDataLen <= link->nfn*5)
 	return CS_BAD_TUPLE;
     for (i = 0; i < link->nfn; i++) {
 	link->fn[i].space = *p; p++;
-	link->fn[i].addr = *(u_int *)p; p += 4;
+	link->fn[i].addr = le32_to_cpu(*(u_int *)p); p += 4;
     }
     return CS_SUCCESS;
 }
@@ -562,7 +623,7 @@ static int parse_longlink_mfc(tuple_t *tuple,
 /*====================================================================*/
 
 static int parse_strings(u_char *p, u_char *q, int max,
-			 char *s, int *ofs, int *found)
+			 char *s, u_char *ofs, u_char *found)
 {
     int i, j, ns;
 
@@ -583,10 +644,7 @@ static int parse_strings(u_char *p, u_char *q, int max,
 	*found = ns;
 	return CS_SUCCESS;
     } else {
-	if (ns == max)
-	    return CS_SUCCESS;
-	else
-	    return CS_BAD_TUPLE;
+	return (ns == max) ? CS_SUCCESS : CS_BAD_TUPLE;
     }
 }
 
@@ -596,7 +654,7 @@ static int parse_vers_1(tuple_t *tuple, cistpl_vers_1_t *vers_1)
 {
     u_char *p, *q;
     
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     q = p + tuple->TupleDataLen;
     
     vers_1->major = *p; p++;
@@ -613,7 +671,7 @@ static int parse_altstr(tuple_t *tuple, cistpl_altstr_t *altstr)
 {
     u_char *p, *q;
     
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     q = p + tuple->TupleDataLen;
     
     return parse_strings(p, q, CISTPL_MAX_ALTSTR_STRINGS,
@@ -627,7 +685,7 @@ static int parse_jedec(tuple_t *tuple, cistpl_jedec_t *jedec)
     u_char *p, *q;
     int nid;
 
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     q = p + tuple->TupleDataLen;
 
     for (nid = 0; nid < CISTPL_MAX_DEVICES; nid++) {
@@ -648,8 +706,8 @@ static int parse_manfid(tuple_t *tuple, cistpl_manfid_t *m)
     if (tuple->TupleDataLen < 4)
 	return CS_BAD_TUPLE;
     p = (u_short *)tuple->TupleData;
-    m->manf = p[0];
-    m->card = p[1];
+    m->manf = le16_to_cpu(p[0]);
+    m->card = le16_to_cpu(p[1]);
     return CS_SUCCESS;
 }
 
@@ -688,7 +746,7 @@ static int parse_config(tuple_t *tuple, cistpl_config_t *config)
     int rasz, rmsz, i;
     u_char *p;
 
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     rasz = *p & 0x03;
     rmsz = (*p & 0x3c) >> 2;
     if (tuple->TupleDataLen < rasz+rmsz+4)
@@ -714,7 +772,6 @@ static int parse_config(tuple_t *tuple, cistpl_config_t *config)
     
 ======================================================================*/
 
-
 static u_char *parse_power(u_char *p, u_char *q,
 			   cistpl_power_t *pwr)
 {
@@ -761,19 +818,22 @@ static u_char *parse_timing(u_char *p, u_char *q,
 	if (++p == q) return NULL;
 	timing->wait = SPEED_CVT(*p);
 	timing->waitscale = exponent[scale & 3];
-    }
+    } else
+	timing->wait = 0;
     scale >>= 2;
     if ((scale & 7) != 7) {
 	if (++p == q) return NULL;
 	timing->ready = SPEED_CVT(*p);
 	timing->rdyscale = exponent[scale & 7];
-    }
+    } else
+	timing->ready = 0;
     scale >>= 3;
     if (scale != 7) {
 	if (++p == q) return NULL;
 	timing->reserved = SPEED_CVT(*p);
 	timing->rsvscale = exponent[scale];
-    }
+    } else
+	timing->reserved = 0;
     p++;
     return p;
 }
@@ -849,7 +909,7 @@ static u_char *parse_mem(u_char *p, u_char *q, cistpl_mem_t *mem)
 	    }
 	mem->win[i].len = len << 8;
 	mem->win[i].card_addr = ca << 8;
-	mem->win[i].host_addr = (caddr_t)(ha << 8);
+	mem->win[i].host_addr = ha << 8;
     }
     return p;
 }
@@ -946,17 +1006,18 @@ static int parse_cftable_entry(tuple_t *tuple,
 	break;
     case 0x20:
 	entry->mem.nwin = 1;
-	entry->mem.win[0].len = (*(u_short *)p) << 8;
+	entry->mem.win[0].len = le16_to_cpu(*(u_short *)p) << 8;
 	entry->mem.win[0].card_addr = 0;
-	entry->mem.win[0].host_addr = NULL;
+	entry->mem.win[0].host_addr = 0;
 	p += 2;
 	if (p > q) return CS_BAD_TUPLE;
 	break;
     case 0x40:
 	entry->mem.nwin = 1;
-	entry->mem.win[0].len = (*(u_short *)p) << 8;
-	entry->mem.win[0].card_addr = (*(u_short *)(p+2)) << 8;
-	entry->mem.win[0].host_addr = (caddr_t)entry->mem.win[0].host_addr;
+	entry->mem.win[0].len = le16_to_cpu(*(u_short *)p) << 8;
+	entry->mem.win[0].card_addr =
+	    le16_to_cpu(*(u_short *)(p+2)) << 8;
+	entry->mem.win[0].host_addr = 0;
 	p += 4;
 	if (p > q) return CS_BAD_TUPLE;
 	break;
@@ -989,10 +1050,10 @@ static int parse_bar(tuple_t *tuple, cistpl_bar_t *bar)
     u_char *p;
     if (tuple->TupleDataLen < 6)
 	return CS_BAD_TUPLE;
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     bar->attr = *p;
     p += 2;
-    bar->size = p[0] + (p[1]<<8) + (p[2]<<16) + (p[3]<<24);
+    bar->size = le32_to_cpu(*(u_int *)p);
     return CS_SUCCESS;
 }
 
@@ -1000,12 +1061,12 @@ static int parse_config_cb(tuple_t *tuple, cistpl_config_t *config)
 {
     u_char *p;
     
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     if ((*p != 3) || (tuple->TupleDataLen < 6))
 	return CS_BAD_TUPLE;
     config->last_idx = *(++p);
     p++;
-    config->base = p[0] + (p[1]<<8) + (p[2]<<16) + (p[3]<<24);
+    config->base = le32_to_cpu(*(u_int *)p);
     config->subtuples = tuple->TupleDataLen - 6;
     return CS_SUCCESS;
 }
@@ -1090,7 +1151,7 @@ static int parse_device_geo(tuple_t *tuple, cistpl_device_geo_t *geo)
     u_char *p, *q;
     int n;
 
-    p = (char *)tuple->TupleData;
+    p = (u_char *)tuple->TupleData;
     q = p + tuple->TupleDataLen;
 
     for (n = 0; n < CISTPL_MAX_DEVICES; n++) {
@@ -1121,7 +1182,7 @@ static int parse_vers_2(tuple_t *tuple, cistpl_vers_2_t *v2)
 
     v2->vers = p[0];
     v2->comply = p[1];
-    v2->dindex = *(u_short *)(p+2);
+    v2->dindex = le16_to_cpu(*(u_short *)(p+2));
     v2->vspec8 = p[6];
     v2->vspec9 = p[7];
     v2->nhdr = p[8];
@@ -1218,6 +1279,10 @@ int parse_tuple(client_handle_t handle, tuple_t *tuple, cisparse_t *parse)
     case CISTPL_ORG:
 	ret = parse_org(tuple, &parse->org);
 	break;
+    case CISTPL_NO_LINK:
+    case CISTPL_LINKTARGET:
+	ret = CS_SUCCESS;
+	break;
     default:
 	ret = CS_UNSUPPORTED_FUNCTION;
 	break;
