@@ -2,7 +2,21 @@
 
     Resource management routines
 
-    rsrc_mgr.c 1.29 1998/01/07 04:44:34 (David Hinds)
+    rsrc_mgr.c 1.49 1998/08/20 10:46:40
+
+    The contents of this file are subject to the Mozilla Public
+    License Version 1.0 (the "License"); you may not use this file
+    except in compliance with the License. You may obtain a copy of
+    the License at http://www.mozilla.org/MPL/
+
+    Software distributed under the License is distributed on an "AS
+    IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
+    implied. See the License for the specific language governing
+    rights and limitations under the License.
+
+    The initial developer of the original code is David A. Hinds
+    <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+    are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
     
 ======================================================================*/
 
@@ -10,6 +24,7 @@
 #define __NO_VERSION__
 #include <pcmcia/k_compat.h>
 
+#ifdef __LINUX__
 #include <linux/sched.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
@@ -17,7 +32,9 @@
 #include <linux/malloc.h>
 #include <linux/ioport.h>
 #include <linux/timer.h>
+#include <asm/irq.h>
 #include <asm/io.h>
+#endif
 
 #include <pcmcia/cs_types.h>
 #include <pcmcia/ss.h>
@@ -29,25 +46,52 @@
 
 /*====================================================================*/
 
-typedef struct memory_entry_t {
-    u_long			base, num;
-    char			*name;
-    struct memory_entry_t	*next;
-} memory_entry_t;
+/* Parameters that can be set with 'insmod' */
 
-/* An ordered linked list of allocated memory blocks */
-static memory_entry_t memlist = { 0, 0, NULL, NULL };
+/* Should we probe resources for conflicts? */
+#ifdef CONFIG_ISA
+static int probe_io = 1;
+static int probe_mem = 1;
+static int mem_limit = 0x10000;
+MODULE_PARM(probe_io, "i");
+MODULE_PARM(probe_mem, "i");
+MODULE_PARM(mem_limit, "i");
+#endif
+
+/*======================================================================
+
+    The resource_entry_t structures are used to track resources used
+    by PC Card drivers.  The resource_map_t structures are used to
+    track what resources are available for allocation.
+
+======================================================================*/
 
 typedef struct resource_entry_t {
     u_long			base, num;
+    char			*name;
     struct resource_entry_t	*next;
 } resource_entry_t;
 
+/* An ordered linked list of allocated memory blocks */
+static resource_entry_t mem_list = { 0, 0, NULL, NULL };
+
+#ifdef __BEOS__
+/* An ordered linked list of allocated IO blocks */
+static resource_entry_t io_list = { 0, 0, NULL, NULL };
+#endif
+
+typedef struct resource_map_t {
+    u_long			base, num;
+    struct resource_map_t	*next;
+} resource_map_t;
+
 /* Memory resource database */
-static resource_entry_t mem_db = { 0, 0, &mem_db };
+static resource_map_t mem_db = { 0, 0, &mem_db };
 
 /* IO port resource database */
-static resource_entry_t io_db = { 0, 0, &io_db };
+static resource_map_t io_db = { 0, 0, &io_db };
+
+#ifdef CONFIG_ISA
 
 typedef struct irq_info_t {
     u_int			Attributes;
@@ -56,37 +100,27 @@ typedef struct irq_info_t {
 } irq_info_t;
 
 /* Table of IRQ assignments */
-static irq_info_t irq_table[16] = { { 0, 0, 0 }, /* etc */ };
+static irq_info_t irq_table[NR_IRQS] = { { 0, 0, 0 }, /* etc */ };
 
-/*====================================================================*/
+#endif
 
-/* Parameters that can be set with 'insmod' */
-
-/* Should we probe resources for conflicts? */
-static int probe_io = 1;
-static int probe_mem = 1;
-static int mem_limit = 0x10000;
-
-MODULE_PARM(probe_io, "i");
-MODULE_PARM(probe_mem, "i");
-MODULE_PARM(mem_limit, "i");
+#ifdef __SMP__
+static spinlock_t rsrc_lock = SPIN_LOCK_UNLOCKED;
+#endif
 
 /*======================================================================
 
-    These functions manage the database of allocated memory blocks.
+    These functions manage the databases of allocated resources
     
 ======================================================================*/
 
-static memory_entry_t *find_gap(memory_entry_t *root,
-				memory_entry_t *entry)
+static resource_entry_t *find_gap(resource_entry_t *root,
+				  resource_entry_t *entry)
 {
-    u_long flags;
-    memory_entry_t *p;
+    resource_entry_t *p;
     
     if (entry->base > entry->base+entry->num-1)
 	return NULL;
-    save_flags(flags);
-    cli();
     for (p = root; ; p = p->next) {
 	if ((p != root) && (p->base+p->num-1 >= entry->base)) {
 	    p = NULL;
@@ -96,72 +130,125 @@ static memory_entry_t *find_gap(memory_entry_t *root,
 	    (p->next->base > entry->base+entry->num-1))
 	    break;
     }
-    restore_flags(flags);
     return p;
 }
 
-int check_mem_region(u_long base, u_long num)
+static int check_resource(resource_entry_t *list,
+			  u_long base, u_long num)
 {
-    memory_entry_t entry;
+    resource_entry_t entry;
+    u_long flags;
+    int ret;
+
     entry.base = base;
     entry.num = num;
-    return (find_gap(&memlist, &entry) == NULL) ? -EBUSY : 0;
+    spin_lock_irqsave(&rsrc_lock, flags);
+    ret = (find_gap(list, &entry) == NULL) ? -EBUSY : 0;
+    spin_unlock_irqrestore(&rsrc_lock, flags);
+    return ret;
 }
 
-int register_mem_region(u_long base, u_long num, char *name)
+static int register_resource(resource_entry_t *list,
+			     u_long base, u_long num, char *name)
 {
     u_long flags;
-    memory_entry_t *p, *entry;
+    resource_entry_t *p, *entry;
 
-    entry = kmalloc(sizeof(memory_entry_t), GFP_ATOMIC);
+    entry = kmalloc(sizeof(resource_entry_t), GFP_ATOMIC);
     entry->base = base;
     entry->num = num;
     entry->name = name;
 
-    save_flags(flags);
-    cli();
-    p = find_gap(&memlist, entry);
+    spin_lock_irqsave(&rsrc_lock, flags);
+    p = find_gap(list, entry);
     if (p == NULL) {
-	restore_flags(flags);
-	kfree_s(entry, sizeof(memory_entry_t));
+	spin_unlock_irqrestore(&rsrc_lock, flags);
+	kfree_s(entry, sizeof(resource_entry_t));
 	return -EBUSY;
     }
     entry->next = p->next;
     p->next = entry;
-    restore_flags(flags);
+    spin_unlock_irqrestore(&rsrc_lock, flags);
     return 0;
 }
 
-int release_mem_region(u_long base, u_long num)
+static int release_resource(resource_entry_t *list,
+			    u_long base, u_long num)
 {
     u_long flags;
-    memory_entry_t *p, *q;
+    resource_entry_t *p, *q;
 
-    save_flags(flags);
-    cli();
-    for (p = &memlist; ; p = q) {
+    spin_lock_irqsave(&rsrc_lock, flags);
+    for (p = list; ; p = q) {
 	q = p->next;
 	if (q == NULL) break;
 	if ((q->base == base) && (q->num == num)) {
 	    p->next = q->next;
-	    kfree_s(q, sizeof(memory_entry_t));
-	    restore_flags(flags);
+	    kfree_s(q, sizeof(resource_entry_t));
+	    spin_unlock_irqrestore(&rsrc_lock, flags);
 	    return 0;
 	}
     }
-    restore_flags(flags);
+    spin_unlock_irqrestore(&rsrc_lock, flags);
     return -EINVAL;
 }
 
+int check_mem_region(u_long base, u_long num)
+{
+    return check_resource(&mem_list, base, num);
+}
+int register_mem_region(u_long base, u_long num, char *name)
+{
+    return register_resource(&mem_list, base, num, name);
+}
+int release_mem_region(u_long base, u_long num)
+{
+    return release_resource(&mem_list, base, num);
+}
+
+#ifdef __BEOS__
+int check_io_region(u_long base, u_long num)
+{
+    return check_resource(&io_list, base, num);
+}
+int register_io_region(u_long base, u_long num, char *name)
+{
+    return register_resource(&io_list, base, num, name);
+}
+int release_io_region(u_long base, u_long num)
+{
+    return release_resource(&io_list, base, num);
+}
+#endif
+
+/*====================================================================*/
+
+#ifdef HAS_PROC_BUS
+int proc_read_mem(char *buf, char **start, off_t pos,
+		  int count, int *eof, void *data)
+{
+    resource_entry_t *r;
+    u_long flags;
+    char *p = buf;
+    
+    spin_lock_irqsave(&rsrc_lock, flags);
+    for (r = mem_list.next; r; r = r->next)
+	p += sprintf(p, "%08lx-%08lx : %s\n", r->base,
+		     r->base+r->num-1, r->name);
+    spin_unlock_irqrestore(&rsrc_lock, flags);
+    return (p - buf);
+}
+#endif
+
 /*======================================================================
 
     These manage the internal databases of available resources
     
 ======================================================================*/
 
-static int add_interval(resource_entry_t *map, u_long base, u_long num)
+static int add_interval(resource_map_t *map, u_long base, u_long num)
 {
-    resource_entry_t *p, *q;
+    resource_map_t *p, *q;
 
     for (p = map; ; p = p->next) {
 	if ((p != map) && (p->base+p->num-1 >= base))
@@ -169,7 +256,7 @@ static int add_interval(resource_entry_t *map, u_long base, u_long num)
 	if ((p->next == map) || (p->next->base > base+num-1))
 	    break;
     }
-    q = kmalloc(sizeof(resource_entry_t), GFP_KERNEL);
+    q = kmalloc(sizeof(resource_map_t), GFP_KERNEL);
     q->base = base; q->num = num;
     q->next = p->next; p->next = q;
     return 0;
@@ -177,9 +264,9 @@ static int add_interval(resource_entry_t *map, u_long base, u_long num)
 
 /*====================================================================*/
 
-static int sub_interval(resource_entry_t *map, u_long base, u_long num)
+static int sub_interval(resource_map_t *map, u_long base, u_long num)
 {
-    resource_entry_t *p, *q;
+    resource_map_t *p, *q;
 
     for (p = map; ; p = q) {
 	q = p->next;
@@ -190,7 +277,7 @@ static int sub_interval(resource_entry_t *map, u_long base, u_long num)
 		if (q->base+q->num <= base+num) {
 		    /* Delete whole block */
 		    p->next = q->next;
-		    kfree_s(q, sizeof(resource_entry_t));
+		    kfree_s(q, sizeof(resource_map_t));
 		    /* don't advance the pointer yet */
 		    q = p;
 		} else {
@@ -203,7 +290,7 @@ static int sub_interval(resource_entry_t *map, u_long base, u_long num)
 		q->num = base - q->base;
 	    } else {
 		/* Split the block into two pieces */
-		p = kmalloc(sizeof(resource_entry_t), GFP_KERNEL);
+		p = kmalloc(sizeof(resource_map_t), GFP_KERNEL);
 		p->base = base+num;
 		p->num = q->base+q->num - p->base;
 		q->num = base - q->base;
@@ -221,6 +308,7 @@ static int sub_interval(resource_entry_t *map, u_long base, u_long num)
     
 ======================================================================*/
 
+#ifdef CONFIG_ISA
 static void do_io_probe(ioaddr_t base, ioaddr_t num)
 {
     
@@ -236,9 +324,9 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     for (i = base; i < base+num; i += 8) {
 	if (check_region(i, 8) != 0)
 	    continue;
-	hole = inb_p(i);
+	hole = inb(i);
 	for (j = 1; j < 8; j++)
-	    if (inb_p(i+j) != hole) break;
+	    if (inb(i+j) != hole) break;
 	if (j == 8)
 	    b[hole]++;
     }
@@ -252,7 +340,7 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
 	if (check_region(i, 8) != 0)
 	    continue;
 	for (j = 0; j < 8; j++)
-	    if (inb_p(i+j) != hole) break;
+	    if (inb(i+j) != hole) break;
 	if (j < 8) {
 	    if (!any)
 		printk(" excluding");
@@ -278,6 +366,7 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     
     printk(any ? "\n" : " clean.\n");
 }
+#endif
 
 /*======================================================================
 
@@ -287,20 +376,19 @@ static void do_io_probe(ioaddr_t base, ioaddr_t num)
     
 ======================================================================*/
 
-#define STEP 8192
-
 static int do_mem_probe(u_long base, u_long num, int pass,
 			int (*is_valid)(u_long))
 {
-    u_long i, j, bad;
+    u_long i, j, bad, step;
 
     printk(KERN_INFO "cs: memory probe 0x%06lx-0x%06lx:",
 	   base, base+num-1);
     if (pass != 0) release_mem_region(base, num);
     bad = 0;
-    for (i = base; i < base+num; i = j + STEP) {
-	for (j = i; j < base+num; j += STEP)
-	    if ((check_mem_region(j, STEP)) == 0 && is_valid(j))
+    step = (num < 0x20000) ? 0x2000 : ((num>>4) & ~0x1fff);
+    for (i = base; i < base+num; i = j + step) {
+	for (j = i; j < base+num; j += step)
+	    if ((check_mem_region(j, step) == 0) && is_valid(j))
 		break;
 	if (i != j) {
 	    if (!bad) printk(" excluding");
@@ -313,22 +401,41 @@ static int do_mem_probe(u_long base, u_long num, int pass,
     return (num - bad);
 }
 
-void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
+#ifdef CONFIG_ISA
+
+void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
+		  int force_low)
 {
-    resource_entry_t *m;
+    resource_map_t *m;
     static u_char order[] = { 0xd0, 0xe0, 0xc0, 0xf0 };
+    static int hi = 0, lo = 0;
     u_long b, ok = 0;
     int i, pass;
     
-    if (!probe_mem)
-	return;
-    /* We do up to two passes through the list */
+    if (!probe_mem) return;
+    /* We do up to four passes through the list */
+    if (!force_low) {
+	if (hi++) return;
+	for (pass = 0; pass < 2; pass++) {
+	    for (m = mem_db.next; m != &mem_db; m = m->next) {
+		/* Only probe > 1 MB */
+		if (m->base < 0x100000) continue;
+		ok += do_mem_probe(m->base, m->num, pass,
+				   pass?do_cksum:is_valid);
+	    }
+	    if (ok) return;
+	}
+	printk(KERN_NOTICE "cs: warning: no high memory space "
+	       "available!\n");
+    }
+    if (lo++) return;
     for (pass = 0; pass < 2; pass++) {
 	for (m = mem_db.next; m != &mem_db; m = m->next) {
 	    /* Only probe < 1 MB */
 	    if (m->base >= 0x100000) continue;
 	    if ((m->base | m->num) & 0xffff) {
-		ok += do_mem_probe(m->base, m->num, pass, is_valid);
+		ok += do_mem_probe(m->base, m->num, pass,
+				   pass?do_cksum:is_valid);
 		continue;
 	    }
 	    /* Special probe for 64K-aligned block */
@@ -338,15 +445,36 @@ void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
 		    if (ok >= mem_limit)
 			register_mem_region(b, 0x10000, "skipped");
 		    else
-			ok += do_mem_probe(b, 0x10000, pass, is_valid);
+			ok += do_mem_probe(b, 0x10000, pass,
+					   pass?do_cksum:is_valid);
+		    
 		}
 	    }
 	}
 	if (ok) return;
-	is_valid = do_cksum;
     }
 }
 
+#else /* CONFIG_ISA */
+
+void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long),
+		  int force_low)
+{
+    resource_map_t *m;
+    static int done = 0;
+    int pass;
+    
+    if (!probe_mem || done++)
+	return;
+    for (pass = 0; pass < 2; pass++) {
+	for (m = mem_db.next; m != &mem_db; m = m->next)
+	    if (do_mem_probe(m->base, m->num, pass,
+			     pass?do_cksum:is_valid)) return;
+    }
+}
+
+#endif /* CONFIG_ISA */
+
 /*======================================================================
 
     These find ranges of I/O ports or memory addresses that are not
@@ -357,7 +485,7 @@ void validate_mem(int (*is_valid)(u_long), int (*do_cksum)(u_long))
 int find_io_region(ioaddr_t *base, ioaddr_t num, char *name)
 {
     ioaddr_t align;
-    resource_entry_t *m;
+    resource_map_t *m;
     
     if (*base != 0) {
 	for (m = io_db.next; m != &io_db; m = m->next) {
@@ -386,11 +514,12 @@ int find_io_region(ioaddr_t *base, ioaddr_t num, char *name)
     return -1;
 } /* find_io_region */
 
-int find_mem_region(u_long *base, u_long num, char *name)
+int find_mem_region(u_long *base, u_long num, char *name,
+		    int force_low)
 {
     u_long align;
-    resource_entry_t *m;
-    
+    resource_map_t *m;
+
     if (*base != 0) {
 	for (m = mem_db.next; m != &mem_db; m = m->next) {
 	    if ((*base >= m->base) && (*base+num <= m->base+m->num))
@@ -400,12 +529,18 @@ int find_mem_region(u_long *base, u_long num, char *name)
     }
     
     for (align = 4096; align < num; align *= 2) ;
-    for (m = mem_db.next; m != &mem_db; m = m->next) {
-	for (*base = (m->base + align - 1) & (~(align-1));
-	     *base+align <= m->base+m->num;
-	     *base += align)
-	    if (register_mem_region(*base, num, name) == 0)
-		return 0;
+
+    while (1) {
+	for (m = mem_db.next; m != &mem_db; m = m->next) {
+	    /* first pass >1MB, second pass <1MB */
+	    if ((force_low != 0) ^ (m->base < 0x100000)) continue;
+	    for (*base = (m->base + align - 1) & (~(align-1));
+		 *base+align <= m->base+m->num; *base += align)
+		if (register_mem_region(*base, num, name) == 0)
+		    return 0;
+	}
+	if (force_low) break;
+	force_low++;
     }
     return -1;
 } /* find_mem_region */
@@ -418,7 +553,11 @@ int find_mem_region(u_long *base, u_long num, char *name)
     
 ======================================================================*/
 
+#ifdef CONFIG_ISA
+
+#ifdef __LINUX__
 static void fake_irq IRQ(int i, void *d, struct pt_regs *r) { }
+#endif
 
 int try_irq(u_int Attributes, int irq, int specific)
 {
@@ -449,9 +588,11 @@ int try_irq(u_int Attributes, int irq, int specific)
     } else {
 	if ((info->Attributes & RES_RESERVED) && !specific)
 	    return CS_IN_USE;
+#ifdef __LINUX__
 	if (REQUEST_IRQ(irq, fake_irq, 0, "bogus", NULL) != 0)
 	    return CS_IN_USE;
 	FREE_IRQ(irq, NULL);
+#endif
 	switch (Attributes & IRQ_TYPE) {
 	case IRQ_TYPE_EXCLUSIVE:
 	    info->Attributes |= RES_ALLOCATED;
@@ -473,8 +614,12 @@ int try_irq(u_int Attributes, int irq, int specific)
     return 0;
 } /* try_irq */
 
+#endif
+
 /*====================================================================*/
 
+#ifdef CONFIG_ISA
+
 void undo_irq(u_int Attributes, int irq)
 {
     irq_info_t *info;
@@ -496,7 +641,9 @@ void undo_irq(u_int Attributes, int irq)
 	break;
     }
 }
-    
+
+#endif
+
 /*======================================================================
 
     The various adjust_* calls form the external interface to the
@@ -509,7 +656,7 @@ static int adjust_memory(adjust_t *adj)
     u_long base, num;
     int i;
 
-    base = (u_long)adj->resource.memory.Base;
+    base = adj->resource.memory.Base;
     num = adj->resource.memory.Size;
     if ((num == 0) || (base+num-1 < base))
 	return CS_BAD_SIZE;
@@ -522,9 +669,10 @@ static int adjust_memory(adjust_t *adj)
     case REMOVE_MANAGED_RESOURCE:
 	sub_interval(&mem_db, base, num);
 	for (i = 0; i < sockets; i++) {
-	    socket_info_t *s = socket_table[i];
-	    release_mem_region(s->cis_mem.sys_start, 0x1000);
-	    s->cis_mem.sys_start = 0;
+	    release_cis_mem(socket_table[i]);
+#ifdef CONFIG_CARDBUS
+	    cb_release_cis_mem(socket_table[i]);
+#endif
 	}
 	break;
     default:
@@ -552,8 +700,10 @@ static int adjust_io(adjust_t *adj)
     case ADD_MANAGED_RESOURCE:
 	if (add_interval(&io_db, base, num) != 0)
 	    return CS_IN_USE;
+#ifdef CONFIG_ISA
 	if (probe_io)
 	    do_io_probe(base, num);
+#endif
 	break;
     case REMOVE_MANAGED_RESOURCE:
 	sub_interval(&io_db, base, num);
@@ -570,6 +720,7 @@ static int adjust_io(adjust_t *adj)
 
 static int adjust_irq(adjust_t *adj)
 {
+#ifdef CONFIG_ISA
     int irq;
     irq_info_t *info;
     
@@ -602,8 +753,8 @@ static int adjust_irq(adjust_t *adj)
 	return CS_UNSUPPORTED_FUNCTION;
 	break;
     }
-    
-    return 0;
+#endif
+    return CS_SUCCESS;
 } /* adjust_irq */
 
 /*====================================================================*/
@@ -631,14 +782,14 @@ int adjust_resource_info(client_handle_t handle, adjust_t *adj)
 
 void release_resource_db(void)
 {
-    resource_entry_t *p, *q;
+    resource_map_t *p, *q;
     
     for (p = mem_db.next; p != &mem_db; p = q) {
 	q = p->next;
-	kfree_s(p, sizeof(resource_entry_t));
+	kfree_s(p, sizeof(resource_map_t));
     }
     for (p = io_db.next; p != &io_db; p = q) {
 	q = p->next;
-	kfree_s(p, sizeof(resource_entry_t));
+	kfree_s(p, sizeof(resource_map_t));
     }
 }
