@@ -1,50 +1,24 @@
 /*
- * yenta.h 1.8 1998/01/16 07:31:25 (David Hinds)
+ * yenta.h 1.12 1998/05/10 11:59:46
+ *
+ * The contents of this file are subject to the Mozilla Public License
+ * Version 1.0 (the "License"); you may not use this file except in
+ * compliance with the License. You may obtain a copy of the License
+ * at http://www.mozilla.org/MPL/
+ *
+ * Software distributed under the License is distributed on an "AS IS"
+ * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
+ * the License for the specific language governing rights and
+ * limitations under the License. 
+ *
+ * The initial developer of the original code is David A. Hinds
+ * <dhinds@hyper.stanford.edu>.  Portions created by David A. Hinds
+ * are Copyright (C) 1998 David A. Hinds.  All Rights Reserved.
  */
 
 #ifndef _LINUX_YENTA_H
 #define _LINUX_YENTA_H
 
-#ifndef PCI_VENDOR_ID_RICOH
-#define PCI_VENDOR_ID_RICOH		0x1180
-#endif
-
-#ifndef PCI_DEVICE_ID_RICOH_RL5C466
-#define PCI_DEVICE_ID_RICOH_RL5C466	0x0466
-#endif
-
-#ifndef PCI_VENDOR_ID_SMC
-#define PCI_VENDOR_ID_SMC		0x10b3
-#endif
-
-#ifndef PCI_DEVICE_ID_SMC_34C90
-#define PCI_DEVICE_ID_SMC_34C90		0xb106
-#endif
-
-#ifndef PCI_DEVICE_ID_CIRRUS_6832
-#define PCI_DEVICE_ID_CIRRUS_6832	0x1110
-#endif
-
-#ifndef PCI_VENDOR_ID_TI
-#define PCI_VENDOR_ID_TI		0x104c
-#endif
-
-#ifndef PCI_DEVICE_ID_TI_1130
-#define PCI_DEVICE_ID_TI_1130		0xac12
-#endif
-
-#ifndef PCI_DEVICE_ID_TI_1131
-#define PCI_DEVICE_ID_TI_1131		0xac15
-#endif
-
-#ifndef PCI_VENDOR_ID_O2_MICRO
-#define PCI_VENDOR_ID_O2_MICRO		0x1217
-#endif
-
-#ifndef PCI_DEVICE_ID_O2_6832
-#define PCI_DEVICE_ID_O2_6832		0x6832
-#endif
-
 /* PCI Configuration Registers */
 
 #define CB_CONTROL_REG_BASE		0x0010
@@ -144,6 +118,6 @@
 
 /* Address bits 31..24 for memory windows for 16-bit cards,
    accessable only by memory mapping the 16-bit register set */
-#define CB_MEM_PAGE(map)		(0x0840 + (map))
+#define CB_MEM_PAGE(map)		(0x40 + (map))
 
 #endif /* _LINUX_YENTA_H */
